@@ -31,21 +31,22 @@ package Lintian::appstream_metadata;
 
 use strict;
 use warnings;
-
-use Moo;
+use autodie qw(open);
 
 use File::Basename qw(basename);
-use XML::Simple qw(:strict);
+use XML::LibXML;
 
-with('Lintian::Check');
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
 
 sub binary {
     my ($self) = @_;
 
     my $pkg = $self->package;
     my $type = $self->type;
-    my $info = $self->info;
-    my $proc = $self->processable;
+    my $processable = $self->processable;
     my $group = $self->group;
 
     my (%desktopfiles, %metainfo, @udevrules);
@@ -53,13 +54,19 @@ sub binary {
     my $modaliases = [];
     if (
         defined(
-            my $dir = $info->index_resolved_path('usr/share/applications/'))
+            my $dir
+              = $processable->installed->resolve_path(
+                'usr/share/applications/'))
     ) {
-        for my $file ($dir->children('breadth-first')) {
+        for my $file ($dir->descendants) {
             $desktopfiles{$file} = 1 if ($file->is_file);
         }
     }
-    if (defined(my $dir = $info->index_resolved_path('usr/share/metainfo/'))) {
+    if (
+        defined(
+            my $dir
+              = $processable->installed->resolve_path('usr/share/metainfo/'))
+    ) {
         for my $file ($dir->children) {
             if ($file->is_file) {
                 $metainfo{$file} = 1;
@@ -67,16 +74,24 @@ sub binary {
             }
         }
     }
-    if (defined(my $dir = $info->index_resolved_path('usr/share/appdata/'))) {
-        for my $file ($dir->children('breadth-first')) {
+    if (
+        defined(
+            my $dir
+              = $processable->installed->resolve_path('usr/share/appdata/'))
+    ) {
+        for my $file ($dir->descendants) {
             if ($file->is_file) {
                 $self->tag(('appstream-metadata-in-legacy-location', $file));
                 $found_modalias|= $self->check_modalias($file, $modaliases);
             }
         }
     }
-    if (defined(my $dir = $info->index_resolved_path('lib/udev/rules.d/'))) {
-        for my $file ($dir->children('breadth-first')) {
+    if (
+        defined(
+            my $dir= $processable->installed->resolve_path('lib/udev/rules.d/')
+        )
+    ) {
+        for my $file ($dir->descendants) {
             push(@udevrules, $file) if ($file->is_file);
         }
     }
@@ -94,47 +109,58 @@ sub binary {
 sub check_modalias {
     my ($self, $metadatafile, $modaliases) = @_;
 
-    my $info = $self->info;
-
     if (!$metadatafile->is_open_ok) {
         # FIXME report this as an error
         return;
     }
-    my $xml = eval {
-        XMLin(
-            $metadatafile->fs_path,
-            ForceArray => ['provides', 'modalias'],
-            KeepRoot => 1,
-            KeyAttr => [],
-        );
-    };
+
+    my $parser = XML::LibXML->new;
+    $parser->set_option('no_network', 1);
+
+    my $doc = eval {$parser->parse_file($metadatafile->unpacked_path);};
     if ($@) {
         $self->tag('appstream-metadata-invalid',
-            basename($metadatafile->fs_path));
+            basename($metadatafile->unpacked_path));
         return 0;
     }
 
-    if (exists $xml->{'application'}) {
-        $self->tag(('appstream-metadata-legacy-format', $metadatafile));
+    return 0
+      unless $doc;
+
+    if ($doc->findnodes('/application')) {
+        $self->tag('appstream-metadata-legacy-format', $metadatafile);
         return 0;
     }
-    if (   exists $xml->{'component'}
-        && exists $xml->{'component'}{'provides'}
-        && exists $xml->{'component'}{'provides'}[0]{'modalias'}) {
-        for (@{$xml->{'component'}{'provides'}[0]{'modalias'}}) {
-            push(@{$modaliases}, $_);
-            if (m/^usb:v[0-9a-f]{4}p[0-9a-f]{4}d/i
-                && !m/^usb:v[0-9A-F]{4}p[0-9A-F]{4}d/) {
-                $self->tag((
-                    'appstream-metadata-malformed-modalias-provide',
-                    $metadatafile,
-                    "include non-valid hex digit in USB matching rule '$_'"
-                ));
-            }
-        }
-        return 1;
+
+    my @provides = $doc->findnodes('/component/provides');
+    return 0
+      unless @provides;
+
+    # take first one
+    my $first = $provides[0];
+    return 0
+      unless $first;
+
+    my @nodes = $first->getChildrenByTagName('modalias');
+    return 0
+      unless @nodes;
+
+    for my $node (@nodes) {
+
+        my $alias = $node->firstChild->data;
+        next
+          unless $alias;
+
+        push(@{$modaliases}, $alias);
+
+        $self->tag('appstream-metadata-malformed-modalias-provide',
+            $metadatafile,
+            "include non-valid hex digit in USB matching rule '$alias'")
+          if $alias =~ /^usb:v[0-9a-f]{4}p[0-9a-f]{4}d/i
+          && $alias !~ /^usb:v[0-9A-F]{4}p[0-9A-F]{4}d/;
     }
-    return 0;
+
+    return 1;
 }
 
 sub provides_user_device {
@@ -177,7 +203,7 @@ sub provides_user_device {
 sub check_udev_rules {
     my ($self, $file, $data) = @_;
 
-    my $fd = $file->open;
+    open(my $fd, '<', $file->unpacked_path);
     my $linenum = 0;
     my $cont;
     my $retval = 0;
