@@ -1,6 +1,7 @@
 # fields/maintainer -- lintian check script (rewrite) -*- perl -*-
 #
-# Copyright (C) 2004 Marc Brockschmidt
+# Copyright © 2004 Marc Brockschmidt
+# Copyright © 2020 Felix Lechner
 #
 # Parts of the code were taken from the old check script, which
 # was Copyright (C) 1998 Richard Braakman (also licensed under the
@@ -28,19 +29,22 @@ use strict;
 use warnings;
 use autodie;
 
+use Lintian::Data;
 use Lintian::Maintainer qw(check_maintainer);
 
+use constant EMPTY => q{};
+
 use Moo;
 use namespace::clean;
 
 with 'Lintian::Check';
 
+my $KNOWN_DISTS = Lintian::Data->new('changes-file/known-dists');
+
 sub source {
     my ($self) = @_;
 
-    my $info = $self->info;
-
-    my $maintainer = $info->unfolded_field('maintainer');
+    my $maintainer = $self->processable->unfolded_field('maintainer');
 
     return
       unless defined $maintainer;
@@ -48,7 +52,34 @@ sub source {
     my $is_list = $maintainer =~ /\@lists(?:\.alioth)?\.debian\.org\b/;
 
     $self->tag('no-human-maintainers')
-      if $is_list && !defined $info->field('uploaders');
+      if $is_list && !defined $self->processable->field('uploaders');
+
+    return;
+}
+
+sub changes {
+    my ($self) = @_;
+
+    my $group = $self->group;
+    return
+      unless defined $group;
+
+    my $source = $group->source;
+    return
+      unless defined $source;
+
+    my $changes_maintainer = $self->processable->unfolded_field('maintainer')
+      // EMPTY;
+    my $changes_distribution
+      = $self->processable->unfolded_field('distribution')// EMPTY;
+
+    my $source_maintainer = $source->unfolded_field('maintainer') // EMPTY;
+
+    # not for derivatives; https://wiki.ubuntu.com/DebianMaintainerField
+    $self->tag('inconsistent-maintainer',
+        $changes_maintainer . ' (changes vs. source) ' .$source_maintainer)
+      unless $changes_maintainer eq $source_maintainer
+      || !$KNOWN_DISTS->known($changes_distribution);
 
     return;
 }
@@ -56,16 +87,18 @@ sub source {
 sub always {
     my ($self) = @_;
 
-    my $info = $self->info;
+    return
+      if $self->type eq 'changes';
 
-    my $maintainer = $info->unfolded_field('maintainer');
+    my $maintainer = $self->processable->unfolded_field('maintainer');
 
     unless (defined $maintainer) {
         $self->tag('no-maintainer-field');
         return;
     }
 
-    check_maintainer($maintainer, 'maintainer');
+    my @tags = check_maintainer($maintainer, 'maintainer');
+    $self->tag(@{$_}) for @tags;
 
     return;
 }
