@@ -25,8 +25,9 @@ use warnings;
 use autodie;
 
 use Moo;
+use namespace::clean;
 
-with('Lintian::Check');
+with 'Lintian::Check';
 
 my $BUILD_PATH_REGEX
   = Lintian::Data->new('files/build-path-regex',qr/~~~~~/,
@@ -36,8 +37,8 @@ sub files {
     my ($self, $file) = @_;
 
     # build directory
-    unless ($self->processable->pkg_src eq 'sbuild'
-        || $self->processable->pkg_src eq 'pbuilder') {
+    unless ($self->processable->source eq 'sbuild'
+        || $self->processable->source eq 'pbuilder') {
 
         foreach my $buildpath ($BUILD_PATH_REGEX->all) {
             my $regex = $BUILD_PATH_REGEX->value($buildpath);
