@@ -1,6 +1,7 @@
 # md5sums -- lintian check script -*- perl -*-
 
-# Copyright (C) 1998 Christian Schwarz and Richard Braakman
+# Copyright © 1998 Christian Schwarz and Richard Braakman
+# Copyright © 2020 Felix Lechner
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +25,11 @@ use strict;
 use warnings;
 use autodie;
 
-use Lintian::Util qw(dequote_name);
+use List::Compare;
+use Path::Tiny;
+use Try::Tiny;
+
+use Lintian::Util qw(read_md5sums drop_relative_prefix);
 
 use Moo;
 use namespace::clean;
@@ -46,7 +51,7 @@ sub files {
       unless $file->is_regular_file;
 
     $self->_set_only_conffiles(0)
-      unless $self->info->is_conffile($file);
+      unless $self->processable->is_conffile($file);
 
     return;
 }
@@ -54,14 +59,15 @@ sub files {
 sub breakdown {
     my ($self) = @_;
 
-    my $control = $self->info->control_index('md5sums');
+    my $control = $self->processable->control->lookup('md5sums');
 
     # Is there an md5sums control file?
     unless ($control) {
 
         # ignore if package contains no files
         return
-          if -z $self->info->lab_data_path('md5sums');
+          if -z path($self->processable->groupdir)->child('md5sums')
+          ->stringify;
 
         $self->tag('no-md5sums-control-file')
           unless $self->only_conffiles;
@@ -70,53 +76,10 @@ sub breakdown {
     return;
 }
 
-sub read_md5sums_file {
-    my ($self, $file) = @_;
-
-    my %checksums;
-
-    # read in md5sums control file
-    my $fd = $file->open;
-
-    while (my $line = <$fd>) {
-
-        chop $line;
-
-        next
-          if $line =~ m/^\s*$/;
-
-        if ($line=~ m{^(\\)?([a-f0-9]+)\s*(?:\./)?(\S.*)$}) {
-
-            my $escaped = $1;
-            my $checksum = $2;
-            my $path = $3;
-
-            if (length($checksum) != 32) {
-                $self->tag('malformed-md5sums-control-file', "line $.");
-
-                next;
-            }
-
-            $path = dequote_name($path)
-              if $escaped;
-
-            $checksums{$path} = $checksum;
-
-            next;
-        }
-
-        $self->tag('malformed-md5sums-control-file', "line $.");
-    }
-
-    close($fd);
-
-    return \%checksums;
-}
-
 sub binary {
     my ($self) = @_;
 
-    my $control = $self->info->control_index('md5sums');
+    my $control = $self->processable->control->lookup('md5sums');
 
     # Is there an md5sums control file?
     return
@@ -134,31 +97,37 @@ sub binary {
     return
       if $control->size == 0;
 
-    my %md5sums = %{$self->read_md5sums_file($control)};
+    my $text = $control->slurp;
+    my ($md5sums, $errors) = read_md5sums($text);
 
-    # iterate over files found in control file
-    for my $file (keys %md5sums) {
+    $self->tag('malformed-md5sums-control-file', $_)for @{$errors};
 
-        my $calculated = $self->info->md5sums->{$file};
-        unless (defined $calculated) {
+    my %noprefix
+      = map { drop_relative_prefix($_) => $md5sums->{$_} } keys %{$md5sums};
 
-            $self->tag('md5sums-lists-nonexistent-file', $file);
-            next;
-        }
+    my @listed = keys %noprefix;
+    my @found = grep { $_->is_file} $self->processable->installed->sorted_list;
 
-        $self->tag('md5sum-mismatch', $file)
-          unless $calculated eq $md5sums{$file};
+    my $lc = List::Compare->new(\@listed, \@found);
+
+    # find files that should exist but do not
+    $self->tag('md5sums-lists-nonexistent-file', $_)for $lc->get_Lonly;
+
+    # find files that should be listed but are not
+    for my $name ($lc->get_Ronly) {
+
+        $self->tag('file-missing-in-md5sums', $name)
+          unless $self->processable->is_conffile($name)
+          || $name =~ m%^var/lib/[ai]spell/.%;
     }
 
-    # iterate over files present in package
-    for my $file (keys %{ $self->info->md5sums }) {
+    # checksum should match for common files
+    for my $name ($lc->get_intersection) {
 
-        next
-          if $md5sums{$file};
+        my $file = $self->processable->installed->lookup($name);
 
-        $self->tag('file-missing-in-md5sums', $file)
-          unless $self->info->is_conffile($file)
-          || $file =~ m%^var/lib/[ai]spell/.%;
+        $self->tag('md5sum-mismatch', $name)
+          unless $file->md5sum eq $noprefix{$name};
     }
 
     return;
