@@ -30,7 +30,6 @@ use autodie;
 
 use Capture::Tiny qw(capture);
 use List::MoreUtils qw(any);
-use Moo;
 use POSIX qw(strftime);
 use Try::Tiny;
 
@@ -39,7 +38,10 @@ use Lintian::Data;
 use Lintian::Relation;
 use Lintian::Util qw(internal_error safe_qx strip);
 
-with('Lintian::Check');
+use Moo;
+use namespace::clean;
+
+with 'Lintian::Check';
 
 # This is a map of all known interpreters.  The key is the interpreter
 # name (the binary invoked on the #! line).  The value is an anonymous
@@ -245,32 +247,32 @@ sub binary {
     my ($self) = @_;
 
     my $pkg = $self->package;
-    my $info = $self->info;
+    my $processable = $self->processable;
 
     my (%executable, %ELF, %scripts, %seen_helper_cmds);
 
     # no dependency for install-menu, because the menu package specifically
     # says not to depend on it.
 
-    foreach my $file ($info->sorted_index) {
+    foreach my $file ($processable->installed->sorted_list) {
         next if not $file->is_file;
         $ELF{$file} = 1 if $file->file_info =~ /^[^,]*\bELF\b/o;
         next unless $file->operm & 0111;
         $executable{$file} = 1;
     }
 
-    my $all_parsed = Lintian::Relation->and($info->relation('all'),
-        $info->relation('provides'),$pkg);
-    my $str_deps = $info->relation('strong');
+    my $all_parsed = Lintian::Relation->and($processable->relation('all'),
+        $processable->relation('provides'),$pkg);
+    my $str_deps = $processable->relation('strong');
 
     my @x11_fonts
       = grep {m,^usr/share/fonts/X11/.*\.(?:afm|pcf|pfa|pfb)(?:\.gz)?$,}
-      $info->sorted_index;
+      $processable->installed->sorted_list;
 
     my %old_versions;
     for my $entry (
-        $info->changelog
-        ? @{$info->changelog->entries}
+        $processable->changelog
+        ? @{$processable->changelog->entries}
         : ()
     ) {
         my $timestamp = $entry->Timestamp // $OLDSTABLE_RELEASE;
@@ -278,9 +280,15 @@ sub binary {
           if $timestamp < $OLDSTABLE_RELEASE;
     }
 
-    for my $filename (sort keys %{$info->scripts}) {
-        my $interpreter = $info->scripts->{$filename}{interpreter};
-        my $calls_env = $info->scripts->{$filename}{calls_env};
+    for my $file ($processable->installed->sorted_list) {
+
+        next
+          unless $file->is_script;
+
+        my $interpreter = $file->script->{interpreter};
+        my $calls_env = $file->script->{calls_env};
+
+        my $filename = $file->name;
         my $path;
         $scripts{$filename} = 1;
 
@@ -296,7 +304,8 @@ sub binary {
 
         # no checks necessary at all for scripts in /usr/share/doc/
         # unless they are examples
-        next if $in_docs and not $in_examples;
+        next
+          if $in_docs && !$in_examples;
 
         my ($base) = $interpreter =~ m,([^/]*)$,;
 
@@ -311,13 +320,13 @@ sub binary {
           if (  $filename =~ m,\.in$,
             and $interpreter =~ m,^(\@|<\<)[A-Z_]+(\@|>\>)$,);
 
-        my $is_absolute = ($interpreter =~ m,^/, or defined $calls_env);
+        my $is_absolute = ($interpreter =~ m,^/, or $calls_env);
 
         # As a special-exception, Policy 10.4 states that Perl scripts must use
         # /usr/bin/perl directly and not via /usr/bin/env, etc.
         $self->script_tag(bad_interpreter_tag_name('/usr/bin/env perl'),
             $filename, '(#!/usr/bin/env perl != /usr/bin/perl)')
-          if defined $calls_env and $interpreter eq 'perl';
+          if $calls_env and $interpreter eq 'perl';
 
         # Skip files that have the #! line, but are not executable and
         # do not have an absolute path and are not in a bin/ directory
@@ -354,8 +363,13 @@ sub binary {
         $self->script_tag('interpreter-not-absolute', $filename,
             "#!$interpreter")
           unless $is_absolute;
+
+        my $bash_completion_regex
+          = qr{^usr/share/bash-completion/completions/.*};
+
         $self->tag('script-not-executable', $filename)
-          unless ($executable{$filename}
+          unless (
+               $executable{$filename}
             or $filename =~ m,^usr/(?:lib|share)/.*\.pm,
             or $filename =~ m,^usr/(?:lib|share)/.*\.py,
             or $filename =~ m,^usr/(?:lib|share)/ruby/.*\.rb,
@@ -365,8 +379,13 @@ sub binary {
             or $filename =~ m,\.ex$,
             or $filename eq 'etc/init.d/skeleton'
             or $filename =~ m,^etc/menu-methods,
-            or $filename =~ m,^etc/X11/Xsession\.d,)
-          or $in_docs;
+            or $filename =~ $bash_completion_regex,
+            or $filename =~ m,^etc/X11/Xsession\.d,
+          )or $in_docs;
+
+        # for bash completion issue this instead
+        $self->tag('bash-completion-with-hashbang', $filename)
+          if $filename =~ $bash_completion_regex;
 
         # Warn about csh scripts.
         $self->tag('csh-considered-harmful', $filename)
@@ -375,22 +394,22 @@ sub binary {
             and $filename !~ m,^etc/csh/login\.d/,)
           and not $in_docs;
 
-        $path = $info->index_resolved_path($filename);
-        next if not $path or not $path->is_open_ok;
+        next
+          unless $file->is_open_ok;
         # Syntax-check most shell scripts, but don't syntax-check
         # scripts that end in .dpatch.  bash -n doesn't stop checking
         # at exit 0 and goes on to blow up on the patch itself.
         if ($base =~ /^$known_shells_regex$/) {
             if (
                     -x $interpreter
-                and not script_is_evil_and_wrong($path)
+                and not script_is_evil_and_wrong($file)
                 and $filename !~ m,\.dpatch$,
                 and $filename !~ m,\.erb$,
                 # exclude some shells. zsh -n is broken, see #485885
                 and $base !~ m/^(?:z|t?c)sh$/
             ) {
 
-                if (check_script_syntax($interpreter, $path)) {
+                if (check_script_syntax($interpreter, $file)) {
                     $self->script_tag('shell-script-fails-syntax-check',
                         $filename);
                 }
@@ -414,7 +433,7 @@ sub binary {
         }
         if ($data) {
             my $expected = $data->[0] . '/' . $base;
-            unless ($interpreter eq $expected or defined $calls_env) {
+            unless ($interpreter eq $expected or $calls_env) {
                 $self->script_tag(bad_interpreter_tag_name($expected),
                     $filename, "(#!$interpreter != $expected)");
             }
@@ -442,7 +461,7 @@ sub binary {
                 # executable).
                 my $interfile = substr $interpreter, 1;
                 $pinter = 1 if $executable{$interfile};
-            } elsif (defined $calls_env) {
+            } elsif ($calls_env) {
                 for my $dir (qw(usr/bin bin)) {
                     if ($executable{"$dir/$interpreter"}) {
                         $pinter = 1;
@@ -462,7 +481,7 @@ sub binary {
             &&!$str_deps->implies(
                 'libperl4-corelibs-perl | perl (<< 5.12.3-7)')
         ) {
-            my $fd = $path->open;
+            open(my $fd, '<', $file->unpacked_path);
             while (<$fd>) {
                 if (
                     m{ (?:do|require)\s+['"] # do/require
@@ -562,42 +581,43 @@ sub binary {
         }
     }
 
-    foreach (keys %executable) {
-        my $index_info = $info->index($_);
+    for my $name (keys %executable) {
+        my $file = $processable->installed->lookup($name);
         my $ok = 0;
-        if ($index_info->is_hardlink) {
+        if ($file->is_hardlink) {
             # We don't collect script information for hardlinks, so check
             # if the target is a script.
-            my $target = $index_info->link_normalized;
-            if (exists $info->scripts->{$target}) {
-                $ok = 1;
-            }
+            my $targetname = $file->link_normalized;
+            my $target = $processable->installed->lookup($targetname);
+
+            $ok = 1
+              if $target->is_script;
         }
 
-        $self->tag('executable-not-elf-or-script', $_)
+        $self->tag('executable-not-elf-or-script', $name)
           unless (
                $ok
-            or $ELF{$_}
-            or $scripts{$_}
-            or $_ =~ m,^usr(?:/X11R6)?/man/,
-            or $_ =~ m/\.exe$/ # mono convention
-            or $_ =~ m/\.jar$/ # Debian Java policy 2.2
+            or $ELF{$name}
+            or $scripts{$name}
+            or $name =~ m,^usr(?:/X11R6)?/man/,
+            or $name =~ m/\.exe$/ # mono convention
+            or $name =~ m/\.jar$/ # Debian Java policy 2.2
           );
     }
 
     # get maintainer scripts
-    my %control = %{$self->info->control_scripts};
+    my @control
+      = grep { $_->is_control } $self->processable->control->sorted_list;
 
     # Handle control scripts.  This is an edited version of the code for
     # normal scripts above, because there were just enough differences to
     # make a shared function awkward.
 
-    my (%added_diversions, %removed_diversions, %dh_cmd_substs);
+    my (%added_diversions, %removed_diversions);
     my $expand_diversions = 0;
-    for my $file (keys %control) {
+    for my $file (@control) {
 
-        my $path = $info->control_index_resolved_path($file);
-        my $interpreter = $control{$file};
+        my $interpreter = $file->control->{interpreter};
 
         $interpreter =~ m|([^/]*)$|;
         my $base = $1;
@@ -653,12 +673,14 @@ sub binary {
             unless (not $data->[1]) {
                 my $depends = Lintian::Relation->new($data->[1]);
                 if ($file eq 'preinst') {
-                    unless ($info->relation('pre-depends')->implies($depends)){
+                    unless ($processable->relation('pre-depends')
+                        ->implies($depends)){
                         $self->tag('preinst-interpreter-without-predepends',
                             "#!$interpreter");
                     }
                 } else {
-                    unless ($info->relation('strong')->implies($depends)) {
+                    unless (
+                        $processable->relation('strong')->implies($depends)) {
                         $self->tag(
                             'control-interpreter-without-depends',
                             "control/$file",
@@ -678,7 +700,8 @@ sub binary {
         $self->tag('csh-considered-harmful', "control/$file")
           if ($base eq 'csh' or $base eq 'tcsh');
 
-        next if not $path or not $path->is_open_ok;
+        next
+          unless $file->is_open_ok;
 
         my $shellscript = $base =~ /^$known_shells_regex$/ ? 1 : 0;
 
@@ -687,7 +710,7 @@ sub binary {
         if ($shellscript) {
             $checkbashisms = $base eq 'sh' ? 1 : 0;
             if ($base eq 'sh' or $base eq 'bash') {
-                if (check_script_syntax("/bin/${base}", $path)) {
+                if (check_script_syntax("/bin/${base}", $file)) {
                     $self->tag('maintainer-shell-script-fails-syntax-check',
                         $file);
                 }
@@ -695,7 +718,7 @@ sub binary {
         }
 
         # now scan the file contents themselves
-        my $fd = $path->open;
+        open(my $fd, '<', $file->unpacked_path);
 
         my (
             $saw_init, $saw_invoke,
@@ -718,20 +741,20 @@ sub binary {
                 $self->tag('maintainer-script-has-unexpanded-debhelper-token',
                     $file);
             }
-            if (/^# Automatically added by (\S+)\s*$/) {
-                my $dh_cmd = $1;
-                # dh_python puts a trailing ":", remove that.
-                $dh_cmd =~ s/:++$//g;
-                $self->tag('debhelper-autoscript-in-maintainer-scripts',
-                    $dh_cmd)
-                  if not $dh_cmd_substs{$dh_cmd}++;
-                $in_automatic_section = 1;
-            }
+
+            $in_automatic_section = 1
+              if /^# Automatically added by \S+\s*$/;
+
             $in_automatic_section = 0
               if $_ eq '# End automatically added section';
 
-            next if m,^\s*$,;  # skip empty lines
-            next if m,^\s*\#,; # skip comment lines
+            # skip empty lines
+            next
+              if m,^\s*$,;
+
+            # skip comment lines
+            next
+              if m,^\s*\#,;
             $_ = remove_comments($_);
 
             # Concatenate lines containing continuation character (\)
@@ -965,18 +988,18 @@ sub binary {
                     $self->tag('multi-arch-same-package-calls-pycompile',
                         "$file:$.")
                       if m/^\s*py3?compile(?:\s|\z)/
-                      and $info->field('multi-arch', 'no') eq 'same';
+                      and $processable->field('multi-arch', 'no') eq 'same';
 
                     if (m,>\s*/etc/inetd\.conf(?:\s|\Z),) {
                         $self->tag('maintainer-script-modifies-inetd-conf',
                             "$file:$.")
-                          unless $info->relation('provides')
+                          unless $processable->relation('provides')
                           ->implies('inet-superserver');
                     }
                     if (m,^\s*(?:cp|mv)\s+(?:.*\s)?/etc/inetd\.conf\s*$,) {
                         $self->tag('maintainer-script-modifies-inetd-conf',
                             "$file:$.")
-                          unless $info->relation('provides')
+                          unless $processable->relation('provides')
                           ->implies('inet-superserver');
                     }
 
@@ -1036,8 +1059,8 @@ sub binary {
                             or m,command\s+.*?$regex,) {
                             $warned{$package} = 1;
                         } elsif (!/\|\|\s*true\b/) {
-                            unless (
-                                $info->relation('strong')->implies($package)) {
+                            unless ($processable->relation('strong')
+                                ->implies($package)) {
                                 my $shortpackage = $package;
                                 $shortpackage =~ s/[ \(].*//;
                                 $self->tag(
@@ -1079,13 +1102,13 @@ m,$LEADIN(?:/usr/bin/)?dpkg\s+--compare-versions\s+.*\b\Q$ver\E(?!\.)\b,
                   if /--group/ && !/--add/;
             }
 
-            my $pdepends = $info->relation('pre-depends');
+            my $pdepends = $processable->relation('pre-depends');
             $self->tag('skip-systemd-native-flag-missing-pre-depends',
                 "$file:$.")
               if m/invoke-rc.d\b.*--skip-systemd-native\b/
               && !$pdepends->implies('init-system-helpers (>= 1.54~)');
 
-            my $depends = $info->relation('depends');
+            my $depends = $processable->relation('depends');
             $self->tag(
                 'missing-versioned-depends-on-init-system-helpers',
                 "$file:$.",
@@ -1298,10 +1321,13 @@ m,$LEADIN(?:/usr/bin/)?dpkg\s+--compare-versions\s+.*\b\Q$ver\E(?!\.)\b,
 
         if ($expand_diversions) {
             $self->tag('diversion-for-unknown-file', $divert, "$script:$line")
-              unless (any { $_ =~ m/$divertrx/ } $info->sorted_index);
+              unless (
+                any { $_ =~ m/$divertrx/ }
+                $processable->installed->sorted_list
+              );
         } else {
             $self->tag('diversion-for-unknown-file', $divert, "$script:$line")
-              unless $info->index($divert);
+              unless $processable->installed->lookup($divert);
         }
     }
 
@@ -1352,7 +1378,7 @@ sub script_is_evil_and_wrong {
     my $i = 0;
     my $var = '0';
     my $backgrounded = 0;
-    my $fd = $path->open;
+    open(my $fd, '<', $path->unpacked_path);
     local $_;
     while (<$fd>) {
         chomp;
@@ -1421,7 +1447,7 @@ sub script_is_evil_and_wrong {
 sub check_script_syntax {
     my ($interpreter, $path) = @_;
 
-    safe_qx($interpreter, '-n', $path->fs_path);
+    safe_qx($interpreter, '-n', $path->unpacked_path);
     return $?;
 }
 
